/***************************************************************************
 *
 * Authors:     Carlos Oscar S. Sorzano (coss@cnb.csic.es)
 *              Roberto Marabini (roberto@mipg.upenn.edu)
 *
 * Unidad de  Bioinformatica of Centro Nacional de Biotecnologia , CSIC
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307  USA
 *
 *  All comments concerning this program package may be sent to the
 *  e-mail address 'xmipp@cnb.csic.es'
 ***************************************************************************/
/*****************************************************************************/
/* APH Files: MRC                                                            */
/*****************************************************************************/

#ifndef _XMIPP_APH3D_HH
#define _XMIPP_APH3D_HH

#include <data/funcs.h>
#include <data/matrix3d.h>
#include <data/geometry.h>

/**@defgroup APH3D APH3D Files
   @ingroup InterfaceLibrary */
//@{
/** APH2D Files.
    This is a class to read/write APH files as produced by the CCP$'s program
    prepmklcf (just the log file).
    @code
   H   K   L      A      P     FOM*100         REJECTS
                                      0   1  0.000     0.0  180.0     0.0   1.0
   0   1   1  6097.2  -90.0 100.0
   0   1   1  9847.4  -90.0 100.0
    @endcode
    First lines are skiped Each line describes a Fourier spot (if indented is
    ignored), the first  three columns are h, k and l, then the magnitude, the
    phase
    */
class APHFile3D
{
public:
    /** APH Filename */
    FileName fn_aph;
    /** Reduce phase accuracy by factor */
    double Reduce_phase_accuracy;
    /** a vector-magnitude */
    double a;
    /** b vector-magnitude */
    double b;
    /** gamma angle */
    double gamma;
    /** c vector-magnitude */
    double c;
    /** resolution */
    double resolution;
    /** Amplitudes_scaled_by */
    double Amp_Scale;
    /** Matrices with the FT spots. */
    Matrix3D<double> spots_abs;
    Matrix3D<double> spots_arg;
    /** Space Group */
    int Space_Group;
    /** Matrix with the spots FOM. */
    Matrix3D<double> FOM;

public:
    /** Read an aph file generated by the CCP4 program prepmklcf.
        First lines are scanned for cell-dimensions and resolution.
        3 matrices are created
        containing the magnitude and phase (spots_abs and spots_arg) and the
        FOMs.
        */
    void read_from_prepmklcf(const FileName &fn);

    /** Empties actual APH structure. */
    void clear();
};
//@}
#endif
